<?php
/*
	Copyright 2009  Jumplane (http://www.jumplane.com)
   
	This file is part of Jumplane Points

    Jumplane Points is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Jumplane Points is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Jumplane Points.  If not, see <http://www.gnu.org/licenses/>.

*/


/**  
* jlPointsClient Class  
*   
* This class encapsulates the access to the JumpLane Points Server API from  
* a client application.  
*  
* Instantiation  
*  
* jlPointsClient requires a valid pair of API keys to be provided as constructor  
* parameters:  
*  
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* </code> 
*  
* In this example, API_KEY and API_SECRET_KEY are the two constants defined in  
* jlpcAuth.php. It is recommended that you edit this file with your own keys,  
* but nothing prevents you from using the key strings directly when instantiating  
* a new jlPointsClient object.  *  
* Once that the jlPointsClient object is instantiated, you can call any of its  
* methods to access the JumpLane Points Server. For example:  
*  
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $user = $jlpc->GetUserInfo("user1");  
* echo (string)$user->id;  
* echo (string)$user->name;  
* echo (int)$user->points;  
* echo (int)$user->pointsActual;  
* </code>  
*  
* Generally speaking, each method in the jlPointsClient class corresponds to one function  
* in the JumpLane Points API, and therefore the rules documented on the JumpLane site for  
* each API function are valid for the corresponding jlPointsClient class method (e.g.  
* error codes, prerequisites, etc.).  
*  
* In addition to the methods for accessing the API, the jlPointsClient class provides  
* some special methods that allows you to obtain more information about the last executed  
* API function.  
*   
* For more information about the Jumplane Points Server API  
* visit the {@link http://en.codex.jumplane.com/index.php/PointsServerApi JumpLane Points Server API} page.  
*  
*/ 

class jlPointsClient { 
	/** @var string Affiliate's API key */    
		protected $ApiKey;  
	/** @var string Affiliate's API secret key */    
		protected $ApiSecretKey;  
	/** @var string Entry point of the API services */   
		protected $BaseUrl;   
	/** @var int Error code returned by the last API call */    
		protected $ErrorCode;   
	/** @var string Error message returned by the last API call */
		protected $ErrorMsg;  
	/** @var int Http status code returned by the last API call */     
		protected $HttpCode;   
	/** @var string Http method used by the last API call */    
		protected $HttpMethod;   
	/** @var string url accessed by the last API call */     
	protected $Url;  
	/** @var string Xml document returned by the last API call */     
	protected $Xml;   
	
	/** @ignore **/     
		const BASE_URL = 'http://apps.jumplane.com/api.php';   
	/** @ignore **/     
		const DEBUG_MODE = false;   

/**    
* Instantiates a new jlPointsClient object.    
*    
* In order to create a new jlPointsClient object, you need to provide a valid pair    
* of API keys.    
*    
* @param string $ApiKey a valid key to access JumpLane Points Server.    
* @param string $ApiSecretKey a valid secret key to access JumpLane Points Server.    
* @param string $BaseUrl an optional url to be used as the JumpLane Points Server API entry point.    
* Ignore this parameter uUnless you have a strong reason for using an alternative url.    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* </code>    
*/    

function jlPointsClient($ApiKey, $ApiSecretKey, $BaseUrl = null) {        
	$this->ApiKey = $ApiKey;        
	$this->ApiSecretKey = $ApiSecretKey;     
	$this->BaseUrl = $BaseUrl;     
	if (is_null($this->BaseUrl)) {       
		$this->BaseUrl = self::BASE_URL;     
		}     
		}  
		
/**
* Registers an action taken by a user, collecting points for that user.    
*    
* @param string $User user id of the user that performed the action    
* @param string $Action name of the action taken by the user    
* @param string $TargetUser optional parameter specifying additional info     
* about the action taken by the user. It could be an email address, a     
* second user name, etc.    
* @return int the number of points collected by the user.    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* $points = $jlpc->CollectPoints("user1","action1");    
* </code>    
*   
* See also: {@link http://en.codex.jumplane.com/index.php/Points.collect Points.collect}    
*/    
function CollectPoints($User, $Action, $TargetUser = null) {
	$url = "points/collect";         
	$data = "userId=$User&actionName=$Action";         
	if (!is_null($TargetUser)) {             
		$data .= "&targetUser=$TargetUser";         
		}        
	$response = $this->Post($url, $data);         
	return (int)$response->data->pointsCollected;     
	}  
/**    
* Creates a new action.    
*    
* @param string $Id name of the new action    
* @param string $Label label of the new action    
* @param string $Points points that will be obtained by any user who performs this action    
* @return action an action object containing information about the new action.    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* $action = $jlpc->CreateAction("action1","label1",10);    
* echo (string)$action->name;    
* echo (string)$action->label;    
* echo (int)$action->points;    
* </code>    
*/    
function CreateAction($Id, $Label, $Points) {         
	$url = "actions/create";         
	$data = "actionId=$Id&actionLabel=$Label&actionPoints=$Points";         
	$response = $this->Post($url, $data);         
	return $response->data->action;     
	}   
/**    
* Creates a new reward.    
*    
* @param string $Id name of the new reward   
* @param string $Label label of the new reward    
* @param string $Points points needed by any user who wants to exchange points by this reward    
* @return reward a reward object containing information about the new reward.    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* $reward = $jlpc->CreateReward("reward1","label1",10);    
* echo (string)$reward->name;    
* echo (string)$reward->label;    
* echo (int)$reward->points;    
* </code>    
*/     
function CreateReward($Id, $Label, $Points) { 
	$url = "rewards/create"; 
	$data = "rewardId=$Id&rewardLabel=$Label&rewardPoints=$Points";
	$response = $this->Post($url, $data);
	return $response->data->reward;
	}   
/**    
* Deletes an existing action.    
*    
* @param string $Id name of the action to be deleted    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* $jlpc->DeleteAction("action1");   
* </code>    
*/    
function DeleteAction($Id) {
	$url = "actions/delete";         
	$data = "actionId=$Id";         
	$this->Post($url, $data);     
	}   
/**    
* Deletes an existing reward.    
*    
* @param string $id name of the reward to be deleted    
*    
* Example:   
*   
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);
* $jlpc->DeleteReward("reward1");   
* </code>
*/    
function DeleteReward($id) {
	$url = "rewards/delete";
	$data = "rewardId=$id"; 
	$this->Post($url, $data);
	}       
/**
* Modifies an existing action.
*    
* @param string $Id name of the action to modify    
* @param string $Label new label for the action    
* @param string $Points points that will be obtained by any user who performs this action    
* @return action an action object containing information about the modified action.    
*    
* Example:    
*    
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);    
* $action = $jlpc->ModifyAction("action1","label1",10);    
* echo (string)$action->name;    
* echo (string)$action->label;   
* echo (int)$action->points;  
* </code>  
*/  
function ModifyAction($Id, $Label, $Points) {
	$url = "actions/modify"; 
	$data = "actionId=$Id&actionLabel=$Label&actionPoints=$Points";  
	$response = $this->Post($url, $data);  
	return $response->data->action;  
	}   
/**
* Modifies an existing reward. 
*  
* @param string $Id name of the reward to modify   
* @param string $Label new label for the reward   
* @param string $Points points needed by any user who wants to exchange points by this reward  
* @return reward a reward object containing information about the modified reward.  
* 
* Example:  
*   
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $reward = $jlpc->ModifyReward("reward1","label1",10); 
* echo (string)$reward->name; 
* echo (string)$reward->label;  
* echo (int)$reward->points; 
* </code>  
*/ 
function ModifyReward($Id, $Label, $Points) { 
	$url = "rewards/modify";
	$data = "rewardId=$Id&rewardLabel=$Label&rewardPoints=$Points";   
	$response = $this->Post($url, $data);
	return $response->data->reward; 
	} 
/**  
* Registers a reward exchanged by a user, decreasing the user points.  
*  
* @param string $User user id of the user that exchanged the points 
* @param string $Reward id of the reward exchanged by the user  
* @return int the number of points exchanged by the user.   
*   
* Example:  
*  
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $points = $jlpc->ExchangePoints("user1","prize1");  
* </code>   
*  
* See also: {@link http://en.codex.jumplane.com/index.php/Rewards.inform Rewards.inform}  
*/  
function ExchangePoints($User, $Reward) {
	$url = "rewards/inform";      
	$data = "userId=$User&rewardId=$Reward";    
	$response = $this->Post($url, $data);    
	return (int)$response->data->pointsReleased;  
	} 
/** 
* Get a list of available actions. For each action, the name, label and points are 
* returned. 
*  
* @return array the list of actions available for this affiliate. 
*   
* Example:  
*  
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $actions = $jlpc->GetActions(); 
* echo (string)$actions[0]->name; 
* echo (string)$actions[0]->label;
* echo (int)$actions[0]->points;
* </code>   
*  
* See also: {@link http://en.codex.jumplane.com/index.php/Actions.get Actions.get}
*/
function GetActions() {   
	$response = $this->Get("actions/get"); 
	return $response->data->actions->action; 
	}  
/** 
* Obtains the last error code returned by the API. 
* 
* @return int the last error code returned by the last call to the API. 
*   
* Example:  
*  
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $actions = $jlpc->GetUserInfo("unknown_user");  
* echo $jlpc->GetErrorCode(); 
* </code>  
*/  
function GetErrorCode() { 
	return $this->ErrorCode;  
	}  
/**  
* Obtains the last error message returned by the API.
*   
* @return string the last error message returned by the last call to the API.   
*   
* Example: 
*   
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $actions = $jlpc->GetUserInfo("unknown_user");  
* echo $jlpc->GetErrorMsg();  
* </code>  
*/
function GetErrorMsg()  {   
	return $this->ErrorMsg;    
	} 
/**  
* Obtains the last http status code returned by the API. 
*  
* @return int the last http status code returned by the last call to the API. 
*   
* Example:  
*   
* <code>
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $actions = $jlpc->GetActions();  
* echo $jlpc->GetHttpCode(); // 200  
* </code>  
*/   
function GetHttpCode()     {      
	return $this->HttpCode;    
	}  
/** 
* Obtains the last http method used by this class when accessing the API. 
*   
* @return string the last http method used by this class when accessing the API. 
*  
* Example:   
*    
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $actions = $jlpc->GetActions(); 
* echo $jlpc->GetHttpMethod(); // "GET"  
* </code> 
*/   
function GetHttpMethod() {     
	return $this->HttpMethod;   
	}  
/** 
* Obtains a list of actions performed, optionally filtered by user and/or action.  
* Each element of this list contains the user id, action name, action label,  
* points earned and the timestamp corresponding to the event.   
*  
* @param string $User an optional user id. When this parameter is present, only  
* those actions performed by the specified user are included in the results.  
* @param string $Action an optional action name. When this parameter is present,  
* only those actions matching the specified action name are included in the results. 
* @param int $Limit an optional integer that allows the caller to limit the number  
* of elements returned in the array.  
* @return array a list of actions performed, optionally filtered according to the 
* given parameters. 
* 
* Example: 
*  
* <code>   
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $actions = $jlpc->GetPoints("user1","action1",10);   
* echo (string)$actions[0]->user; 
* echo (string)$actions[0]->action;  
* echo (string)$actions[0]->label;  
* echo (int)$actions[0]->points; 
* echo (string)$actions[0]->createdAt; 
* </code> 
* 
* See also: {@link http://en.codex.jumplane.com/index.php/Points.get Points.get} 
*/    
function GetPoints($User = null, $Action = null, $Limit = null) {      
	$command = "points/get";     
	if (!is_null($User)) {  
		$command .= "/userId/$User";   
		}     
	if (!is_null($Action)) {  
		$command .= "/actionName/$Action";    
		}       
	if (!is_null($Limit)) {
		
		$command .= "/limit/$Limit";        
		}        
		$response = $this->Get($command);   
	return $response->data->points->point;     } 
/**   
* Obtains a list of users, sorted by points in descending order. The results can  
* be filtered by a given action by providing a valid action name.   
* Each element of this list contains the position of the user in the ranking, the   
* user name, and the total amount of points for that user.   
*  
* @param string $Action an optional action name. When this parameter is present,    
* only those actions matching the specified action name are taken into account 
* when the ranking is created. 
* @param int $Limit an optional integer that allows the caller to limit the number   
* of elements returned in the array. 
* @return array a list of users sorted by points in descending order, optionally   
* filtered according to the given parameters.    
* 
* Example:   
*  
* <code>    
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $users = $jlpc->GetRanking("action1",10); 
* echo (int)$users[0]->position; 
* echo (string)$users[0]->name;
* echo (int)$users[0]->points;  
* </code>   
*   
* See also: {@link http://en.codex.jumplane.com/index.php/Ranking.get Rankings.get}    
*/    
function GetRanking($Action = null, $Limit = null) {   
	$command = "rankings/get";
	if (!is_null($Action)) {          
		$command .= "/actionName/$Action";   
		}       
	if (!is_null($Limit)) {    
		$command .= "/limit/$Limit";    
		}       
	$response = $this->Get($command);      
	return $response->data->users->user;   
	}    
/**  
* Get a list of available rewards. For each reward, the name, label and points are  
* returned.   
*   
* @return array the list of rewards available for this affiliate.
*  
* Example:  
*  
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $rewards = $jlpc->GetRewards(); 
* echo (string)$rewards[0]->name;  
* echo (string)$rewards[0]->label; 
* echo (int)$rewards[0]->points;  
* </code>  
*  
* See also: {@link http://en.codex.jumplane.com/index.php/Rewards.get Rewards.get} 
*/   
function GetRewards() { 
	$response = $this->Get("rewards/get"); 
	return $response->data->rewards->reward;  
	}   
/**   
* Obtains a list of rewards exchanged, optionally filtered by user and/or reward.  
* Each element of this list contains the user id, reward name, reward label, 
* points exchanged and the timestamp corresponding to the event.   
*  
* @param string $User an optional user id. When this parameter is present, only     
* those events performed by the specified user are included in the results. 
* @param string $Reward an optional reward id. When this parameter is present,  
* only those rewards matching the specified reward id are included in the results.  
* @param int $Limit an optional integer that allows the caller to limit the number  
* of elements returned in the array. 
* @return array a list of rewards exchanged, optionally filtered according to the  
* given parameters. 
*   
* Example:  
* 
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $rewards = $jlpc->GetRewardsHistory("user1","prize1",10);    
* echo (string)$rewards[0]->user; 
* echo (string)$rewards[0]->name;
* echo (string)$rewards[0]->label;  
* echo (int)$rewards[0]->points; 
* echo (string)$rewards[0]->createdAt;  
* </code> 
*   
* See also: {@link http://en.codex.jumplane.com/index.php/Rewards.history Rewards.history} 
*/  
function GetRewardsHistory($User = null, $Reward = null, $Limit = null) {   
	$command = "rewards/history";    
	if (!is_null($User)) {  
		$command .= "/userId/$User"; 
		}    
	if (!is_null($Reward)) {
		$command .= "/rewardId/$Reward";       
			}        
	if (!is_null($Limit)) {       
		$command .= "/limit/$Limit";        
		}        
		$response = $this->Get($command);   
		return $response->data->rewards->reward;   
 }   
 /**  
 * Obtains the last url accessed by this class.  
 *  
 * @return string the last url accessed by this class. 
 *  
 * Example:   
 * 
 * <code> 
 * $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);   
 * $actions = $jlpc->GetActions(); 
 * echo $jlpc->GetUrl(); 
 * </code> 
 */   
 function GetUrl() {   
	 return $this->Url; 
	 }  
/** 
* Obtains information about the specified user.  
*   
* Receives a user id and returns the following information about that user: id, name,  
* total points, actual points, number of rewards exchanged, creation date and if the    
* user is leader or not.  
*   
* @param string $UserId a valid user id. 
* @return user a user object containing information about the specified user.  
*  
* Example:
* 
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $user = $jlpc->GetUserInfo("user1",10);  
* echo (string)$user->id; 
* echo (string)$user->name;
* echo (int)$user->points;   
* echo (int)$user->pointsActual; 
* echo (int)$user->rewards; 
* echo (int)$user->userLeader; 
* echo (string)$user->createdAt; 
* </code>  
*  
* See also: {@link http://en.codex.jumplane.com/index.php/Users.getInfo Users.getInfo}    
*/   
function GetUserInfo($UserId) {   
	$response = $this->Get("users/get_info/userId/$UserId"); 
	return $response->data->user;     
	} 
/**  
* Obtains a list of actions performed by a user. 
*   
* Each element of this list contains the user id, action name, action label, 
* points earned and the timestamp corresponding to the event.  
* 
* @param string $User a valid user id. Only those actions performed by the   
* specified user are included in the results.  
* @param int $Limit an optional integer that allows the caller to limit the number 
* of elements returned in the array.  
* @return array a list of actions performed by the specified user.  
*   
* Example:  
*  
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);  
* $actions = $jlpc->GetUserPoints("user1",10);
* echo (string)$actions[0]->user; 
* echo (string)$actions[0]->action;    
* echo (string)$actions[0]->label;  
* echo (int)$actions[0]->points;   
* echo (string)$actions[0]->createdAt;   
* </code> 
*    
* See also: {@link http://en.codex.jumplane.com/index.php/Users.getPoints Users.getPoints}  
*/  
function GetUserPoints($User, $Limit = null) { 
	$command = "users/get_points/userId/$User";   
	if (!is_null($Limit)) {         
		$command .= "/limit/$Limit";  
	}      
	$response = $this->Get($command);  
	return $response->data->points->point;  
	}  
/**  
* Obtains a list of rewards exchanged by a user. 
*   
* Each element of this list contains the user id, reward name, reward label, 
* points exchanged and the timestamp corresponding to the event. 
*   
* @param string $User a valid user id. Only those rewards exchanged by the 
* specified user are included in the results.   
* @param int $Limit an optional integer that allows the caller to limit the number  
* of elements returned in the array.  
* @return array a list of rewards exchanged by the specified user. 
*  
* Example:   
*  
* <code>  
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY);
* $rewards = $jlpc->GetUserRewards("user1",10); 
* echo (string)$rewards[0]->user;  
* echo (string)$rewards[0]->name;  
* echo (string)$rewards[0]->label;  
* echo (int)$rewards[0]->points; 
* echo (string)$rewards[0]->createdAt;  
* </code> 
* 
* See also: {@link http://en.codex.jumplane.com/index.php/Users.getRewards Users.getRewards} 
*/   
function GetUserRewards($User, $Limit = null) {  
	$command = "users/get_rewards/userId/$User";   
	if (!is_null($Limit)) { 
		$command .= "/limit/$Limit";    
	}     
	$response = $this->Get($command);   
	return $response->data->rewards->reward; 
	}  
/**  
* Obtains the last xml document returned by the API 
*   
* @return string the xml document returned by the last call to the API  
* 
* Example:
* 
* <code> 
* $jlpc = new jlPointsClient(API_KEY, API_SECRET_KEY); 
* $actions = $jlpc->GetActions();   
* echo $jlpc->GetXml();    
* </code>
 */   
 function GetXml() { 
	 return $this->Xml; 
	 }  
/**  
* Sends an Http GET request to the specified url. 
* 
* Typically you will not call this method directly, as it will be called automatically by 
* other methods of this class. 
* 
* @param string $Command url to which the request will be sent, relative to the base url. 
* @return string An object representing the xml document returned by the server.  
*/
function Get($Command) {  
	return $this->SendRequest($Command, 'GET'); 
	}      
/**   
* Sends an Http POST request to the specified url.  
*    
* Typically you will not call this method directly, as it will be called automatically by  
* other methods of this class. 
*   
* @param string $Command url to which the request will be sent, relative to the base url.  
* @param string $PostFields optional fields to be posted to the specified url.  
* @return string An object representing the xml document returned by the server.  
*/   
function Post($Command, $PostFields = null) {     
	return $this->SendRequest($Command, 'POST', $PostFields);  
	}   
/**   
* Sends an Http request to the specified url.  
*   
* Typically you will not call this method directly, as it will be called automatically by 
* other methods of this class.  
*   
* @param string $Command url to which the request will be sent, relative to the base url. 
* @param string $HttpMethod http method to be used (GET, POST, PUT, etc). 
* @param string $PostFields optional fields to be posted to the specified url. 
* @return string An object representing the xml document returned by the server.  
*/ 
protected function SendRequest($Command, $HttpMethod, $PostFields = null) {
   $this->Url = $this->UrlFor($Command);     
   $this->HttpMethod = $HttpMethod;      
   $handle = curl_init($this->Url); 
   $headers[] = "Authorization: Basic ".base64_encode($this->ApiKey.":".$this->ApiSecretKey); 
   curl_setopt($handle, CURLOPT_HTTPHEADER, $headers);
   if ($this->HttpMethod == 'POST') { 
	   curl_setopt($handle, CURLOPT_POST, 1);  
          curl_setopt($handle, CURLOPT_POSTFIELDS, $PostFields);
		  }     
	curl_setopt($handle, CURLOPT_RETURNTRANSFER, 1);    
	$this->Xml = curl_exec($handle);     
	$this->ErrorCode = curl_errno($handle);   
	$this->ErrorMsg = curl_error($handle);   
	$this->HttpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);  
	curl_close($handle);
	if (self::DEBUG_MODE) {   
		$curl = "curl -X $HttpMethod $this->Url -u $this->ApiKey:$this->ApiSecretKey";  
		if ($PostFields) {        
			$curl .= " -d $PostFields";     
			}          
		echo "\n$curl\nResult: $this->HttpCode\n$this->Xml\n";   
	}                 
	if ($this->ErrorCode == 0) {       
		try {       
			$xml = new SimpleXMLElement($this->Xml);     
			$this->ErrorCode = (int)$xml->status;    
			$this->ErrorMsg = (string)$xml->message; 
			} catch (Exception $e) {  
				$this->ErrorCode = -1000;    
				$this->ErrorMsg = $e->getMessage();     
				}    
				} else {   
					$xml = null;   
					}    
					return $xml;  
					}  
/**  
* Converts a relative url into an absolute one.
*  
* Typically you will not call this method directly, as it will be called automatically by 
* other methods of this class.  
*  
* @param string $Command relative url to be converted.  
* @return string An absolute url corresponding to the relative url that was received.  
*/    
protected function UrlFor($Command) {      
	return $this->BaseUrl."/$Command"; 
	} 
} 